
unit brick;
interface

uses
	ctypes;

{$IFDEF FPC}
{$PACKRECORDS C}
{$ENDIF}


	{
		The Brick Engine.

		Copyright (c) 2006, 2007, 2008, 2009, 2010, 2011 Stephen M. Havelka
		Permission is hereby granted, free of charge, to any person
		obtaining a copy of this software and associated documentation
		files (the "Software"), to deal in the Software without
		restriction, including without limitation the rights to use,
		copy, modify, merge, publish, distribute, sublicense, and/or sell
		copies of the Software, and to permit persons to whom the
		Software is furnished to do so, subject to the following
		conditions:

		The above copyright notice and this permission notice shall be
		included in all copies or substantial portions of the Software.

		THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
		EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
		OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
		NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
		HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
		WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
		FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
		OTHER DEALINGS IN THE SOFTWARE.

		For more information, please visit the project web site:
		http://rs.tc/br/
	}

	const
		BRICK_VERSION = '5.4';
	{ Compile-time options - change to reflect your own build }
		ACCEL_BLIT = 1;
		WITH_GL = 1;
		WITH_SIMD = 1;
		WITH_IMAGE = 1;
		WITH_SDL13 = 0;
		MAX_WIDTH = 640;
		MAX_HEIGHT = 480;
	{ maximum motion-control code length }
		MAX_MCP_LENGTH = 384;
	{ maximum on-screen display text length	}
		MAX_STRING_LENGTH = 240;
	{ additional frame definitions }
		DISPL_SPAN = 2;
	{ how many characters in a font? }
		FONT_CT = 128;
	{ how many tiles per map? }
		MAX_TILES = 4096;
	{ maximum convolution-kernel size }
		MAX_CK_SIZE = 7;
	{ how many joysticks? axes, hats, and buttons perjoystick? }
		MAX_JOY = 8;
		MAX_AXES = 8;
		MAX_HATS = 4;
		MAX_BUTTONS = 20;
	{ how many mice? mouse buttons? }
		MAX_MOUSE = 1;
		MAX_MOUSE_BUTTONS = 8;
	{ list defines }
		LIST_HEAD = 0;
		LIST_TAIL = 1;
		LIST_ALL = 2;
	{ audio and video defines }
		GRAPHICS_SDL = 0;
		GRAPHICS_ACCEL = 1;
		GRAPHICS_WINDOWED = 0;
		GRAPHICS_FS = 2;
		GRAPHICS_0 = 0;
		GRAPHICS_90 = 1;
		GRAPHICS_180 = 2;
		GRAPHICS_270 = 3;
		AUDIO_OFF = 0;
		AUDIO_SPEAKER = 1;
		AUDIO_RATE = 22050;
		AUDIO_FORMAT = 8;
		AUDIO_CHANNELS = 2;
		AUDIO_BUFFERS = 512;
	{ frame types }
		FRAME_NONE = 0;
		FRAME_RGBA = 1;
		FRAME_RGB = 2;
		FRAME_HL = 3;
		FRAME_SL = 4;
		FRAME_BR = 5;
		FRAME_CT = 6;
		FRAME_SAT = 7;
		FRAME_DISPL = 8;
		FRAME_CONVO = 9;
		FRAME_LUT = 10;
		FRAME_XOR = 11;
	{ frame effect types }
		FRAME_EFFECT_DROP_SHADOW = 1;
	{ tile-animation types }
		TILE_STILL = 0;
		TILE_FWD = 1;
		TILE_REV = 2;
		TILE_PP = 3;
		TILE_PP_REV = 4;
	{ collision types }
		COLLISION_OFF = 0;
		COLLISION_BOX = 1;
		COLLISION_PIXEL = 2;
	{ collision-related defines }
		COLLISION_ATSTART = -(1);
		COLLISION_NEVER = 0;
		COLLISION_INMOTION = 1;
	{ inspection-related defines }
		INSPECT_NW = 0;
		INSPECT_N = 1;
		INSPECT_NE = 2;
		INSPECT_E = 3;
		INSPECT_SE = 4;
		INSPECT_S = 5;
		INSPECT_SW = 6;
		INSPECT_W = 7;
	{ constants used by the input routines }
		IO_UP = 0;
		IO_RIGHT = 1;
		IO_DOWN = 2;
		IO_LEFT = 3;
		IO_AXIS = 0;
		IO_HAT = 1;
		IO_BUTTON = 2;
	{ graphics-related defines. }
		RGB_BITS = 24;
		RGB_BYTES = 3;
		RGBA_BITS = 32;
		RGBA_BYTES = 4;
		RGB_RANGE = 256;
		RGB_MAX = RGB_RANGE-1;
	{ error codes - generic	}
		ERR = -(1);
	{ error codes - graphics and audio }
		ERR_CANT_REOPEN = -(10);
		ERR_BAD_MODE = -(11);
		ERR_BAD_FILE = -(12);
		ERR_SDL_FAILED = -(13);
		ERR_SDL_VIDEO_FAILED = -(14);
		ERR_SDL_MIXER_FAILED = -(15);
	{ error codes - sprites	}
		ERR_BAD_FRAME_TYPE = -(30);
	{ error codes - loading external data }
		ERR_INVALID_FILE = -(40);
	{ error codes - motion control programs	}
		ERR_BAD_INST = -(51);
		ERR_BAD_VAR = -(52);
		ERR_BAD_ARG = -(53);
		ERR_TOO_LONG = -(54);
		ERR_BAD_LIST = -(60);
		ERR_BAD_INST_BC = -(61);
		ERR_BAD_ARG_BC = -(62);


	{ Pointers to C types }
	Type
		PLongint = ^Longint;
		PSmallInt = ^SmallInt;
		PByte = ^Byte;
		PWord = ^Word;
		PDWord = ^DWord;
		PDouble = ^Double;

	{ basic brick engine data types }
		PBrElement = ^TBrElement;
		TBrElement = record
			data : pointer;
			next : PBrElement;
			prev : PBrElement;
			end;

		PBrList = ^TBrList;
		TBrList = record
			head : PBrElement;
			tail : PBrElement;
			end;

		PBrSortcmp = ^TBrSortcmp;
		TBrSortcmp = function (a:pointer; b:pointer):cint; cdecl;

		PBrDir = ^TBrDir;
		TBrDir = record
			horiz : cint;
			vert : cint;
			end;

		PBrBox = ^TBrBox;
		TBrBox = record
			x1 : cint;
			y1 : cint;
			x2 : cint;
			y2 : cint;
			end;

		PBrPoint = ^TBrPoint;
		TBrPoint = record
			x : cint;
			y : cint;
			end;

		PBrVector = ^TBrVector;
		TBrVector = record
			x : cint;
			y : cint;
			end;

		PBrDimensions = ^TBrDimensions;
		TBrDimensions = record
			w : cint;
			h : cint;
			end;

		PBrClipRect = ^TBrClipRect;
		TBrClipRect = record
			x : cint;
			y : cint;
			w : cint;
			h : cint;
			end;

		PBrConvolution = ^TBrConvolution;
		TBrConvolution = record
			kw : cint;
			kh : cint;
			kernel : array[0..(MAX_CK_SIZE*MAX_CK_SIZE)-1] of cchar;
			divisor : cint;
			offset : cint;
			end;

		PBrLut = ^TBrLut;
		TBrLut = record
			r : array[0..RGB_MAX] of cchar;
			g : array[0..RGB_MAX] of cchar;
			b : array[0..RGB_MAX] of cchar;
			end;

		PBrColor = ^TBrColor;
		TBrColor = record
			r : cuchar;
			g : cuchar;
			b : cuchar;
			a : cuchar;
			end;

		{ a frame of graphics or filter data, as seen in sprites and map tiles }
		PBrFrame = ^TBrFrame;
		TBrFrame = record
			w : cint;
			h : cint;
			tag : cint;
			offset : TBrPoint;
			clip_rect : TBrBox;
			data : pointer;
			aux : pointer;
			scratch : pointer;
			mask : pointer;
			end;

		{ a container for sprite motion-control programs }
		PBrMotion = ^TBrMotion;
		TBrMotion = record
			code : ^cuchar;
			tick : cint;
			end;

		{ the sprite's composited frame	}
		PBrFramearray = ^TBrFramearray;
		TBrFramearray = record
			len : cint;
			arr : ^PBrFrame;
			end;

		{ the sprite struct itself: current frame, position, velocity, frames of graphics data, etc. }
		PBrSprite = ^TBrSprite;
		TBrSprite = record
			cur_frame : cint;
			collides : cint;
			z_hint : cint;
			scale : cint;
			frame_ct : cint;
			frames : PBrFramearray;
			pos : TBrPoint;
			vel : TBrVector;
			bound : PBrBox;
			bc : TBrBox;
			mcp : TBrMotion;
			end;

		{ the tile: animation type, count, current animation index, and collision mode }
		PPBrTile = ^PBrTile;
		PBrTile = ^TBrTile;
		TBrTile = record
			anim_type : cint;
			frame_ct : cint;
			cur_frame : cint;
			collides : cint;
			frames : ^PBrFrame;
			end;

		{ a map has a width and height, tile width and height, a tile index array, and a cell word-array }
		PBrMap = ^TBrMap;
		TBrMap = record
			tw : cint;
			th : cint;
			tiles : array[0..(MAX_TILES)-1] of PBrTile;
			w : cint;
			h : cint;
			data : ^cshort;
			tw_div : pointer;
			th_div : pointer;
			end;

		{ a layer is a camera position, sprite list, map, string list, and some flags. }
		PBrLayer = ^TBrLayer;
		TBrLayer = record
			camera : TBrPoint;
			view : TBrBox;
			sprites : PBrList;
			map : PBrMap;
			strings : PBrList;
			visible : cint;
			sorted : cint;
			end;

		{ the font definition }
		PBrFont = ^TBrFont;
		TBrFont = record
			name : array[0..49] of cchar;
			w : cint;
			h : cint;
			chars : array[0..(FONT_CT)-1] of PBrFrame;
			end;

		{ text strings have a font, position, and a text definition }
		PBrString = ^TBrString;
		TBrString = record
			font : array[0..49] of cchar;
			x : cint;
			y : cint;
			text : array[0..(MAX_STRING_LENGTH)-1] of cchar;
			end;

		{ the result of sprite->map collisions:	did a collision occur?	how far did the sprite move before/after collision? }
		PBrMapCollision = ^TBrMapCollision;
		TBrMapCollision = record
			mode : cint;
			stop : TBrVector;
			go : TBrVector;
			end;

		{ sprite->sprite collisions:	did a collision occur?	in which direction was the sprite moving when it did?	how far did it go? }
		PBrSpriteCollision = ^TBrSpriteCollision;
		TBrSpriteCollision = record
			mode : cint;
			dir : TBrVector;
			stop : TBrVector;
			target : pointer;
			end;

		{ a map fragment, returned by the introspection routines }
		PBrMapFragment = ^TBrMapFragment;
		TBrMapFragment = record
			w : cint;
			h : cint;
			tiles : ^cshort;
			end;

		{ a sound requires two buffers - one for SDL_mixer, and a secondary buffer to hold the sound if it is loaded as raw data }
		PBrSound = ^TBrSound;
		TBrSound = record
			wave : pointer;
			buf : ^cuchar;
			end;

		{ the input struct:
			- axes are analog (or mapped keys) and read in pairs
			- hats are digital and always have a direction
			- some input buttons
			- and some system keys }
		PBrInput = ^TBrInput;
		TBrInput = record
			axis : array[0..(MAX_AXES)-1] of cint;
			hat : array[0..(MAX_HATS)-1] of TBrDir;
			button : array[0..(MAX_BUTTONS)-1] of cint;
			space : cint;
			tab : cint;
			sel : cint;
			pause : cint;
			esc : cint;
			end;

		{ the mouse input struct:	x and y movement since last mouse-read, and any button presses }
		PBrMouse = ^TBrMouse;
		TBrMouse = record
			x : cint;
			y : cint;
			button : array[0..6] of cint;
			end;

		{ the scheduled-event callback type }
		PBrEvent = ^TBrEvent;
		TBrEvent = procedure (data:pointer); cdecl;


	{ The Brick API }

	function GraphicsOpen(w:cint; h:cint; zf:cint; rot:cint; flags:cint):cint; cdecl; external 'br' name 'graphics_open';
	procedure GraphicsClose; cdecl; external 'br' name 'graphics_close';

	function AudioOpen(mode:cint):cint; cdecl; external 'br' name 'audio_open';
	procedure AudioClose; cdecl; external 'br' name 'audio_close';

	function IoFetch(num:cint; result:PBrInput):cint; cdecl; external 'br' name 'io_fetch';
	function IoMouse(num:cint; result:PBrMouse):cint; cdecl; external 'br' name 'io_mouse';
	procedure IoGrab(mode:cint); cdecl; external 'br' name 'io_grab';
	function IoHasQuit:cint; cdecl; external 'br' name 'io_has_quit';
	function IoWait(delay:cint):cint; cdecl; external 'br' name 'io_wait';
	procedure IoAssign(number:cint; input:cint; args:array of const); cdecl; external 'br' name 'io_assign';
	function IoReadKey:cint; cdecl; external 'br' name 'io_read_key';

	procedure RenderSetBgFill(fill:cint); cdecl; external 'br' name 'render_set_bg_fill';
	procedure RenderSetBgColor(r:cuchar; g:cuchar; b:cuchar); cdecl; external 'br' name 'render_set_bg_color';
	procedure RenderSetOverdraw(w:cint; h:cint); cdecl; external 'br' name 'render_set_overdraw';
	procedure RenderDisplay; cdecl; external 'br' name 'render_display';
	function RenderToDisk(const filename:PChar):cint; cdecl; external 'br' name 'render_to_disk';

	procedure FontAdd(const name:PChar; w:cint; h:cint; data:pcuchar; widths:array of cint); cdecl; external 'br' name 'font_add';
	procedure FontFromDisk(const name:PChar; const filename:PChar; widths:array of cint); cdecl; external 'br' name 'font_from_disk';
	procedure FontFromBuffer(const name:PChar; len:cint; data:pcuchar; widths:array of cint); cdecl; external 'br' name 'font_from_buffer';

	function SoundLoadFromDisk(const filename:PChar):PBrSound; cdecl; external 'br' name 'sound_load_from_disk';
	function SoundLoadFromBuffer(len:cint; const data:pcuchar):PBrSound; cdecl; external 'br' name 'sound_load_from_buffer';
	function SoundLoadRaw(len:cint; const data:pcuchar):PBrSound; cdecl; external 'br' name 'sound_load_raw';
	function SoundPlay(sound:PBrSound; vol:cint):cint; cdecl; external 'br' name 'sound_play';
	procedure SoundHalt(channel:cint); cdecl; external 'br' name 'sound_halt';
	procedure SoundAdjustVol(channel:cint; vol:cint); cdecl; external 'br' name 'sound_adjust_vol';
	procedure SoundAdjustPan(channel:cint; pan:cint); cdecl; external 'br' name 'sound_adjust_pan';

	procedure SongPlayFromDisk(const filename:PChar; fade:cint); cdecl; external 'br' name 'song_play_from_disk';
	procedure SongPlayFromBuffer(len:cint; const data:pcuchar; fade:cint); cdecl; external 'br' name 'song_play_from_buffer';
	procedure SongStop(fade:cint); cdecl; external 'br' name 'song_stop';
	procedure SongPause; cdecl; external 'br' name 'song_pause';
	procedure SongResume; cdecl; external 'br' name 'song_resume';
	procedure SongSetPosition(tick:cint); cdecl; external 'br' name 'song_set_position';
	procedure SongAdjustVol(vol:cint); cdecl; external 'br' name 'song_adjust_vol';

  { Items and lists, etc }

	function ListCreate:PBrList; cdecl; external 'br' name 'list_create';
	procedure ListEmpty(list:PBrList); cdecl; external 'br' name 'list_empty';
	procedure ListDelete(list:PBrList); cdecl; external 'br' name 'list_delete';
	procedure ListAdd(list:PBrList; item:pointer); cdecl; external 'br' name 'list_add';
	procedure ListPrepend(list:PBrList; item:pointer); cdecl; external 'br' name 'list_prepend';
	function ListShift(list:PBrList):pointer; cdecl; external 'br' name 'list_shift';
	function ListPop(list:PBrList):pointer; cdecl; external 'br' name 'list_pop';
	procedure ListRemove(list:PBrList; item:pointer; dir:cint); cdecl; external 'br' name 'list_remove';
	function ListLength(list:PBrList):cint; cdecl; external 'br' name 'list_length';
	function ListFind(list:PBrList; item:pointer):cint; cdecl; external 'br' name 'list_find';
	procedure ListSort(list:PBrList; compare:PBrSortcmp); cdecl; external 'br' name 'list_sort';

	function LayerCount:cint; cdecl; external 'br' name 'layer_count';
	function LayerAdd:cint; cdecl; external 'br' name 'layer_add';
	procedure LayerReorder(layer:cint; layer2:cint); cdecl; external 'br' name 'layer_reorder';
	procedure LayerRemove(layer_id:cint); cdecl; external 'br' name 'layer_remove';
	function LayerCopy(layer:cint):cint; cdecl; external 'br' name 'layer_copy';
	function LayerGetSpriteList(layer:cint):PBrList; cdecl; external 'br' name 'layer_get_sprite_list';
	function LayerGetMap(layer:cint):PBrMap; cdecl; external 'br' name 'layer_get_map';
	function LayerGetStringList(layer:cint):PBrList; cdecl; external 'br' name 'layer_get_string_list';
	function LayerGetVisible(layer:cint):cint; cdecl; external 'br' name 'layer_get_visible';
	function LayerGetSorting(layer:cint):cint; cdecl; external 'br' name 'layer_get_sorting';
	function LayerGetView(layer:cint; view:PBrBox):cint; cdecl; external 'br' name 'layer_get_view';
	procedure LayerSetSpriteList(layer:cint; list:PBrList); cdecl; external 'br' name 'layer_set_sprite_list';
	procedure LayerSetMap(layer:cint; map:PBrMap); cdecl; external 'br' name 'layer_set_map';
	procedure LayerSetStringList(layer:cint; list:PBrList); cdecl; external 'br' name 'layer_set_string_list';
	procedure LayerSetVisible(layer:cint; visible:cint); cdecl; external 'br' name 'layer_set_visible';
	procedure LayerSetSorting(layer:cint; sorting:cint); cdecl; external 'br' name 'layer_set_sorting';
	procedure LayerSetView(layer:cint; view:PBrBox); cdecl; external 'br' name 'layer_set_view';
	function LayerGetCamera(layer:cint; x:pcint; y:pcint):cint; cdecl; external 'br' name 'layer_get_camera';
	procedure LayerSetCamera(layer:cint; x:cint; y:cint); cdecl; external 'br' name 'layer_set_camera';
	procedure LayerAdjustCamera(layer:cint; x:cint; y:cint); cdecl; external 'br' name 'layer_adjust_camera';

	function FrameInfo(frame:PBrFrame; w:pcint; h:pcint; tag:pcint):cint; cdecl; external 'br' name 'frame_info';
	function FrameCreate(mode:cint; w:cint; h:cint; const data:pointer; const aux:pointer):PBrFrame; cdecl; external 'br' name 'frame_create';
	function FrameCopy(frame:PBrFrame):PBrFrame; cdecl; external 'br' name 'frame_copy';
	procedure FrameDelete(frame:PBrFrame); cdecl; external 'br' name 'frame_delete';
	procedure FrameSetOffset(frame:PBrFrame; x:cint; y:cint); cdecl; external 'br' name 'frame_set_offset';
	procedure FrameSetMask(frame:PBrFrame; const data:pcuchar); cdecl; external 'br' name 'frame_set_mask';
	procedure FrameSetMaskFrom(frame:PBrFrame; src:PBrFrame); cdecl; external 'br' name 'frame_set_mask_from';
	function FrameSlice(frame:PBrFrame; x:cint; y:cint; w:cint; h:cint):PBrFrame; cdecl; external 'br' name 'frame_slice';
	function FrameConvert(frame:PBrFrame; mode:cint; const aux:pointer):PBrFrame; cdecl; external 'br' name 'frame_convert';
	function FrameEffect(frame:PBrFrame; mode:cint; args:array of const):PBrFrame; cdecl; external 'br' name 'frame_effect';
	function FrameFromDisk(const filename:PChar; key:PBrColor):PBrFrame; cdecl; external 'br' name 'frame_from_disk';
	function FrameFromBuffer(len:cint; const data:pcuchar; key:PBrColor):PBrFrame; cdecl; external 'br' name 'frame_from_buffer';

	function MapCreate:PBrMap; cdecl; external 'br' name 'map_create';
	procedure MapEmpty(map:PBrMap, delete_tiles:cint); cdecl; external 'br' name 'map_empty';
	procedure MapDelete(map:PBrMap); cdecl; external 'br' name 'map_delete';
	function MapGetTileSize(map:PBrMap; tw:pcint; th:pcint):cint; cdecl; external 'br' name 'map_get_tile_size';
	function MapGetSize(map:PBrMap; w:pcint; h:pcint):cint; cdecl; external 'br' name 'map_get_size';
	function MapGetTile(map:PBrMap; index:cint; tile:PPBrTile):cint; cdecl; external 'br' name 'map_get_tile';
	procedure MapSetTileSize(map:PBrMap; tw:cint; th:cint); cdecl; external 'br' name 'map_set_tile_size';
	procedure MapSetSize(map:PBrMap; w:cint; h:cint); cdecl; external 'br' name 'map_set_size';
	procedure MapSetTile(map:PBrMap; index:cint; tile:PBrTile); cdecl; external 'br' name 'map_set_tile';
	procedure MapSetData(map:PBrMap; const data:pcshort); cdecl; external 'br' name 'map_set_data';
	procedure MapSetSingle(map:PBrMap; x:cint; y:cint; data:cshort); cdecl; external 'br' name 'map_set_single';
	procedure MapAnimateTiles(map:PBrMap); cdecl; external 'br' name 'map_animate_tiles';
	procedure MapResetTiles(map:PBrMap); cdecl; external 'br' name 'map_reset_tiles';

	function TileCreate:PBrTile; cdecl; external 'br' name 'tile_create';
	procedure TileDelete(tile:PBrTile); cdecl; external 'br' name 'tile_delete';
	function TileAddFrame(tile:PBrTile; frame:PBrFrame):cint; cdecl; external 'br' name 'tile_add_frame';
	function TileAddFrameData(tile:PBrTile; mode:cint; w:cint; h:cint; const data:pointer; const aux:pointer):cint; cdecl; external 'br' name 'tile_add_frame_data';
	function TileGetCollides(tile:PBrTile; collides:pcint):cint; cdecl; external 'br' name 'tile_get_collides';
	function TileGetAnimType(tile:PBrTile; animtype:pcint):cint; cdecl; external 'br' name 'tile_get_anim_type';
	procedure TileSetCollides(tile:PBrTile; collides:cint); cdecl; external 'br' name 'tile_set_collides';
	procedure TileSetAnimType(tile:PBrTile; animtype:cint); cdecl; external 'br' name 'tile_set_anim_type';
	procedure TileSetPixelMask(tile:PBrTile; index:cint; const data:pcuchar); cdecl; external 'br' name 'tile_set_pixel_mask';
	procedure TileSetPixelMaskFrom(tile:PBrTile; index:cint; frame:PBrFrame); cdecl; external 'br' name 'tile_set_pixel_mask_from';
	procedure TileAnimate(tile:PBrTile); cdecl; external 'br' name 'tile_animate';
	procedure TileReset(tile:PBrTile); cdecl; external 'br' name 'tile_reset';

	function SpriteCreate:PBrSprite; cdecl; external 'br' name 'sprite_create';
	function SpriteCopy(sprite:PBrSprite):PBrSprite; cdecl; external 'br' name 'sprite_copy';
	procedure SpriteDelete(sprite:PBrSprite); cdecl; external 'br' name 'sprite_delete';
	function SpriteGetFrame(sprite:PBrSprite; index:pcint):cint; cdecl; external 'br' name 'sprite_get_frame';
	function SpriteGetZHint(sprite:PBrSprite; zhint:pcint):cint; cdecl; external 'br' name 'sprite_get_z_hint';
	function SpriteGetCollides(sprite:PBrSprite; collides:pcint):cint; cdecl; external 'br' name 'sprite_get_collides';
	function SpriteGetPosition(sprite:PBrSprite; x:pcint; y:pcint):cint; cdecl; external 'br' name 'sprite_get_position';
	function SpriteGetVelocity(sprite:PBrSprite; x:pcint; y:pcint):cint; cdecl; external 'br' name 'sprite_get_velocity';
	function SpriteGetScale(sprite:PBrSprite; x:pcint; y:pcint):cint; cdecl; external 'br' name 'sprite_get_scale';
	procedure SpriteSetFrame(sprite:PBrSprite; frame:cint); cdecl; external 'br' name 'sprite_set_frame';
	procedure SpriteSetZHint(sprite:PBrSprite; zhint:cint); cdecl; external 'br' name 'sprite_set_z_hint';
	procedure SpriteSetCollides(sprite:PBrSprite; collides:cint); cdecl; external 'br' name 'sprite_set_collides';
	procedure SpriteSetPosition(sprite:PBrSprite; x:cint; y:cint); cdecl; external 'br' name 'sprite_set_position';
	procedure SpriteSetVelocity(sprite:PBrSprite; x:cint; y:cint); cdecl; external 'br' name 'sprite_set_velocity';
	procedure SpriteSetScale(sprite:PBrSprite; x:cint; y:cint); cdecl; external 'br' name 'sprite_set_scale';
	procedure SpriteSetBoundingBox(sprite:PBrSprite; index:cint; bounds:PBrBox); cdecl; external 'br' name 'sprite_set_bounding_box';
	procedure SpriteSetPixelMask(sprite:PBrSprite; index:cint; const data:pcuchar); cdecl; external 'br' name 'sprite_set_pixel_mask';
	procedure SpriteSetPixelMaskFrom(sprite:PBrSprite; index:cint; frame:PBrFrame); cdecl; external 'br' name 'sprite_set_pixel_mask_from';
	function SpriteAddFrame(sprite:PBrSprite; frame:PBrFrame):cint; cdecl; external 'br' name 'sprite_add_frame';
	function SpriteAddFrameData(sprite:PBrSprite; mode:cint; w:cint; h:cint; const data:pointer; const aux:pointer):cint; cdecl; external 'br' name 'sprite_add_frame_data';
	function SpriteAddSubframe(sprite:PBrSprite; index:cint; frame:PBrFrame):cint; cdecl; external 'br' name 'sprite_add_subframe';
	function SpriteLoadProgram(sprite:PBrSprite; const mcp:PChar):cint; cdecl; external 'br' name 'sprite_load_program';

	function StringCreate:PBrString; cdecl; external 'br' name 'string_create';
	procedure StringDelete(stg:PBrString); cdecl; external 'br' name 'string_delete';
	function StringGetBox(stg:PBrString; w:pcint; h:pcint):cint; cdecl; external 'br' name 'string_get_box';
	procedure StringSetFont(stg:PBrString; const fontname:PChar); cdecl; external 'br' name 'string_set_font';
	procedure StringSetPosition(stg:PBrString; x:cint; y:cint); cdecl; external 'br' name 'string_set_position';
	procedure StringSetText(stg:PBrString; const text:PChar); cdecl; external 'br' name 'string_set_text';

  { Collision and introspection }

	procedure inspectAdjacentTiles(map:PBrMap; sprite:PBrSprite; slip:cint; result:PBrMapFragment); cdecl; external 'br' name 'inspect_adjacent_tiles';
	procedure inspectObscuredTiles(map:PBrMap; sprite:PBrSprite; result:PBrMapFragment); cdecl; external 'br' name 'inspect_obscured_tiles';
	function InspectLineOfSight(map:PBrMap; sprite:PBrSprite; xofs:cint; yofs:cint; dist:cint; tgt:PBrSprite):cint; cdecl; external 'br' name 'inspect_line_of_sight';
	function InspectInFrame(list:PBrList; bounds:PBrBox):PBrList; cdecl; external 'br' name 'inspect_in_frame';
	function InspectNearPoint(list:PBrList; x:cint; y:cint; dist:cint):PBrList; cdecl; external 'br' name 'inspect_near_point';

	procedure CollisionWithMap(sprite:PBrSprite; map:PBrMap; slip:cint; result:PBrMapCollision); cdecl; external 'br' name 'collision_with_map';
	function CollisionWithSprites(sprite:PBrSprite; list:PBrList; limit:cint; result:PBrSpriteCollision):cint; cdecl; external 'br' name 'collision_with_sprites';

  { Motion control programs }

	function MotionExecSingle(sprite:PBrSprite):cint; cdecl; external 'br' name 'motion_exec_single';
	function MotionExecList(list:PBrList):cint; cdecl; external 'br' name 'motion_exec_list';

  { Utilities }

	function ClockMs:cint; cdecl; external 'br' name 'clock_ms';
	function ClockWait(fps:cint):cint; cdecl; external 'br' name 'clock_wait';

	function EventAdd(delay:cint; ct:cint; ev:PBrEvent; data:pointer):cint; cdecl; external 'br' name 'event_add';
	procedure EventMessage(id:cint; msg:cint); cdecl; external 'br' name 'event_message';

	procedure InitBrick; cdecl; external 'br' name 'init_brick';
	procedure QuitBrick; cdecl; external 'br' name 'quit_brick';

implementation


end.
