#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-collision.h"




/* find map collisions */
int wrap_collision_map(lua_State *l)
{
	sprite *sprite;
	map *map;
	int slip;

	/* the results */
	map_collision res;


	/* get the sprite */
	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	/* and the map */
	map = lua_touserdata(l, 2);
	if( !map )
		return luaL_error(l, "Invalid map ");

	/* and the optional slip argument */
	if( lua_gettop(l) == 3 )
		slip = lua_tonumber(l, 3);
	else
		slip = 0;

	/* call the c function */
	collision_with_map(sprite, map, slip, &res);

	/* and return the results */
	lua_pushinteger(l, res.mode);
	lua_pushinteger(l, res.stop.x);
	lua_pushinteger(l, res.stop.y);
	lua_pushinteger(l, res.go.x);
	lua_pushinteger(l, res.go.y);
	return 5;

}


/* find sprite collisions: [ [ target mode dir.x dir.y stop.x stop.y ] .. ] */
int wrap_collision_sprites(lua_State *l)
{
	/* sprite pointer, sprite list pointer */
	sprite *sprite;
	list *my_l;

	/* results */
	int i, ct;
	sprite_collision cols[MAX_SPRITE_COLLISIONS];


	/* get the sprite */
	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	/* and the target list */
	my_l = lua_touserdata(l, 2);
	if( !my_l )
		return luaL_error(l, "Invalid target list ");


	/* call the c function */
	ct = collision_with_sprites(sprite, my_l, MAX_SPRITE_COLLISIONS, cols);

	/* produce the results list */
	for(i=0; i < ct; i++)
		{
			lua_newtable(l);

			/* build the table of result vectors and the target */
			lua_pushinteger(l, 1);
			lua_pushinteger(l, cols[i].mode);
			lua_settable(l, -3);

			lua_pushinteger(l, 2);
			lua_pushlightuserdata(l, cols[i].target);
			lua_settable(l, -3);

			lua_pushinteger(l, 3);
			lua_pushinteger(l, cols[i].dir.x);
			lua_settable(l, -3);

			lua_pushinteger(l, 4);
			lua_pushinteger(l, cols[i].dir.y);
			lua_settable(l, -3);

			lua_pushinteger(l, 5);
			lua_pushinteger(l, cols[i].stop.x);
			lua_settable(l, -3);

			lua_pushinteger(l, 6);
			lua_pushinteger(l, cols[i].stop.y);
			lua_settable(l, -3);

		}

	/* return the results */
	return ct;

}
