#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-font.h"








/* add a new font */
int wrap_font_add(lua_State *l)
{
	const char *name;
	int w, h;                  /* char width, height, and count of bytes in the bytearray */
	const unsigned char *data; /* the bytearray */
	size_t len;

	/* for proportional-width fonts */
	int i, widths[FONT_CT];

	name = lua_tostring(l, 1);
	w = lua_tonumber(l, 2);
	h = lua_tonumber(l, 3);
	data = lua_tolstring(l, 4, &len);

	if( lua_istable(l, 5) )
		{
			/* they've given a set of widths */
			lua_pushnil(l);
			for( i=0; lua_next(l, 5) && i < FONT_CT; i++ )
				{
					widths[i] = lua_tointeger(l, -1);
					lua_pop(l, 1);
				}

			/* and add the font */
			font_add(name, w, h, data, widths);

		}
	else
		/* just add the font */
		font_add(name, w, h, data, NULL);


	return 0;

}




/* we have been requested to load a font from a file */
int wrap_font_from_disk(lua_State *l)
{
	/* source filename */
	const char *name, *file;

	/* for proportional-width fonts */
	int i, widths[FONT_CT];

	/* read arguments */
	name = lua_tostring(l, 1);
	file = lua_tostring(l, 2);

	if( lua_istable(l, 3) )
		{
			/* they've given a set of widths */
			lua_pushnil(l);
			for( i=0; lua_next(l, 3) && i < FONT_CT; i++ )
				{
					widths[i] = lua_tointeger(l, -1);
					lua_pop(l, 1);
				}

			/* and load the font with widths */
			font_from_disk(name, file, widths);

		}
	else
		/* just load the font */
		font_from_disk(name, file, NULL);

	return 0;

}


/* we have been requested to load a font from a memory buffer */
int wrap_font_from_buffer(lua_State *l)
{
	/* source buffer */
	const char *name;
	const unsigned char *data;
	size_t len;

	/* for proportional-width fonts */
	int i, widths[FONT_CT];

	/* read arguments */
	name = lua_tostring(l, 1);
	data = lua_tolstring(l, 2, &len);

	if( lua_istable(l, 3) )
		{
			/* they've given a set of widths */
			lua_pushnil(l);
			for( i=0; lua_next(l, 3) && i < FONT_CT; i++ )
				{
					widths[i] = lua_tointeger(l, -1);
					lua_pop(l, 1);
				}

			/* and load the font with widths */
			font_from_buffer(name, len, data, widths);

		}
	else
		/* just load the font */
		font_from_buffer(name, len, data, NULL);

	return 0;
}
