#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-graphics.h"





/* process the request to open a graphics mode .. */
int wrap_graphics_open(lua_State *l)
{
	/* screen dimensions, flags for full-screen, output filter, zoom factor */
	const char *orient, *opt;
	int w, h, zf, rot, flags;
	int res;

	w = lua_tonumber(l, 1);
	h = lua_tonumber(l, 2);
	zf = lua_tonumber(l, 3);
	orient = lua_tostring(l, 4);
	flags = 0;


	/* parse out the options */
	if( !orient )
		rot = GRAPHICS_0;
	else if( !strcmp(orient, "90") )
		rot = GRAPHICS_90;
	else if( !strcmp(orient, "180") )
		rot = GRAPHICS_180;
	else if( !strcmp(orient, "270") )
		rot = GRAPHICS_270;
	else
		rot = GRAPHICS_0;



	/* has the programmer passed in any optional args? */
	if( lua_gettop(l) == 5 )
		{
			lua_pushnil(l);
			while( lua_next(l, 5) )
				{
					opt = lua_tostring(l, -1);

					/* parse out the options */
					if( !strcmp(opt, "sdl") )
						flags |= GRAPHICS_SDL;
					else if( !strcmp(opt, "accel") )
						flags |= GRAPHICS_ACCEL;
					else if( !strcmp(opt, "windowed") )
						flags |= GRAPHICS_WINDOWED;
					else if( !strcmp(opt, "fs") )
						flags |= GRAPHICS_FS;
					else
						return luaL_error(l, "Unknown graphics option ");

					lua_pop(l, 1);
				}

		}

	/* now open the graphics */
	res = graphics_open(w, h, zf, rot, flags);


	/* if the result was an error, pass back an error code to lua */
	if( res < 0)
		{
			switch(res)
				{
					case ERR_CANT_REOPEN: return luaL_error(l, "A graphics mode is already active ");
					case ERR_BAD_MODE: return luaL_error(l, "No valid mode specified ");
					case ERR_SDL_FAILED: return luaL_error(l, "Could not initialize SDL video subsystem ");
					case ERR_SDL_VIDEO_FAILED: return luaL_error(l, "Could not open the video display ");
				}

		}

	/* otherwise, we have had success! */
	return 0;

}

/* they are requesting to open a graphics mode .. */
int wrap_graphics_close(lua_State *l)
{
	graphics_close();
	return 0;
}

/* a request to set the window title */
int wrap_graphics_window_title(lua_State *l)
{
	const char *title;
	title = lua_tostring(l, 1);
	SDL_WM_SetCaption(title,NULL);
	return 0;
}

