#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-io.h"




/*
 * fetch the current io status and return to lua:
 * [axes ] [ hats ] [ buttons ] space tab sel pause esc
 *    0        1         2        3    4   5    6    7
*/
int wrap_io_fetch(lua_State *l)
{
	int num;

	/* the result vars */
	int i;
	input io;

	/* fetch the input to read */
	num = lua_tonumber(l, 1);


	/* here, note that we offset the input by one for the result table */
	if( !io_fetch(num, &io) )
		{
			/* load the axes */
			lua_newtable(l);
			for(i=0; i < MAX_AXES; i++)
				{
					lua_pushinteger(l, i+1);
					lua_pushinteger(l, io.axis[i]);
					lua_settable(l, -3);
				}

			/* and the hats */
			lua_newtable(l);
			for(i=0; i < MAX_HATS; i++)
				{
					lua_pushinteger(l, i*2+1);
					lua_pushinteger(l, io.hat[i].x);
					lua_settable(l, -3);

					lua_pushinteger(l, i*2+2);
					lua_pushinteger(l, io.hat[i].y);
					lua_settable(l, -3);
				}

			/* and the buttons */
			lua_newtable(l);
			for(i=0; i < MAX_BUTTONS; i++)
				{
					lua_pushinteger(l, i+1);
					lua_pushinteger(l, io.button[i]);
					lua_settable(l, -3);
				}


			lua_pushboolean(l, io.space);
			lua_pushboolean(l, io.tab);
			lua_pushboolean(l, io.sel);
			lua_pushboolean(l, io.pause);
			lua_pushboolean(l, io.esc);
			return 8;

		}
	else
		return luaL_error(l, "Not valid input ");

}






/*
 * fetch the mouse input and return:
 * x y [ buttons ]
 */
int wrap_io_mouse(lua_State *l)
{
	int num;

	/* the result vars */
	int i;
	mouse m;


	/* fetch the input to read */
	num = lua_tonumber(l, 1);

	/* here, note that we offset the input by one for the result table */
	if( !io_mouse(num, &m))
		{
			lua_pushinteger(l, m.x);
			lua_pushinteger(l, m.y);

			/* and the buttons */
			lua_newtable(l);
			for(i=0; i < MAX_MOUSE_BUTTONS; i++)
				{
					lua_pushinteger(l, i);
					lua_pushinteger(l, m.button[i]);
					lua_settable(l, -3);
				}

			return 3;

		}
	else
		return luaL_error(l, "Not valid mouse ");

}




/* return the status of the application-quit button */
int wrap_io_has_quit(lua_State *l)
{
	lua_pushboolean(l, io_has_quit());
	return 1;
}




/* enable or disable the input-grab */
int wrap_io_grab(lua_State *l)
{
	io_grab(lua_tonumber(l, 1));
	return 0;
}




/* wait for the input buffer to be cleared, and then set */
int wrap_io_wait(lua_State *l)
{
	io_wait(lua_tonumber(l, 1));
	return 0;
}



/* assign a keypress to an input action */
int wrap_io_assign(lua_State *l)
{
	/*
	 * the various user-provided options:
	 * - input number
	 * - type of input (axis, hat, button)
	 * - which one of that input type
	 * - if axis or hat, the direction to reassign
	 * - the new key to reassign
	 */
	int num, idx;
	const char *type, *dir;
	int key;


	num = lua_tonumber(l, 1);
	type = lua_tostring(l, 2);

	/* fetch the args appropriate to the type */
	if( !strcmp(type, "axis") )
		{
			idx = lua_tonumber(l, 3);
			dir = lua_tostring(l, 4);
			key = lua_tonumber(l, 5);

			/* map the direction onto the C constants */
			if( !strcmp(dir, "left") )
				io_assign(num, IO_AXIS, idx, IO_LEFT, key);
			else if( !strcmp(dir, "right") )
				io_assign(num, IO_AXIS, idx, IO_RIGHT, key);
			else
				return luaL_error(l, "Unknown direction ");

		}
	else if( strcmp(type, "hat") )
		{
			idx = lua_tonumber(l, 3);
			dir = lua_tostring(l, 4);
			key = lua_tonumber(l, 5);

			/* map the direction onto the C constants */
			if( !strcmp(dir, "left") )
				io_assign(num, IO_HAT, idx, IO_LEFT, key);
			else if( !strcmp(dir, "up") )
				io_assign(num, IO_HAT, idx, IO_UP, key);
			else if( !strcmp(dir, "right") )
				io_assign(num, IO_HAT, idx, IO_RIGHT, key);
			else if( !strcmp(dir, "down") )
				io_assign(num, IO_HAT, idx, IO_DOWN, key);
			else
				return luaL_error(l, "Unknown direction ");

		}
	else if( strcmp(type, "button") )
		{
			idx = lua_tonumber(l, 3);
			key = lua_tonumber(l, 4);

			io_assign(num, IO_BUTTON, idx, key);
		}

	return 0;

}



/* read one key and return the keycode, for use with ::io::assign */
int wrap_io_read_key(lua_State *l)
{
	lua_pushinteger(l, io_read_key());
	return 1;
}
