#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-layer.h"





/* adding a new layer .. */
int wrap_layer_add(lua_State *l)
{
	lua_pushinteger(l, layer_add());
	return 1;
}



/* swapping two layers */
int wrap_layer_swap(lua_State *l)
{
	layer_reorder(lua_tonumber(l, 1), lua_tonumber(l, 2));
	return 0;
}



/* removing a layer */
int wrap_layer_remove(lua_State *l)
{
	layer_remove(lua_tonumber(l, 1));
	return 0;
}



/* making a copy of a layer */
int wrap_layer_copy(lua_State *l)
{
	int id, new;

	id = lua_tonumber(l, 1);

	/* make the copy and catch potential errors */
	new = layer_copy(id);
	if( new == ERR )
		return luaL_error(l, "Not a valid layer ");

	lua_pushinteger(l, new);
	return 1;
}



/* set the layer's sprite list */
int wrap_layer_sprite_list(lua_State *l)
{
	int id;

	/* setting or reading the sprite list? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 2 )
		{
			layer_set_sprite_list(id, lua_touserdata(l, 2));
			return 0;
		}
	else
		{
			lua_pushlightuserdata(l, layer_get_sprite_list(id));
			return 1;
		}

}



/* set the layer's map */
int wrap_layer_map(lua_State *l)
{
	int id;
	map *map;

	/* setting or reading the map? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 2 )
		{
			layer_set_map(id, lua_touserdata(l, 2));
			return 0;
		}
	else
		{
			lua_pushlightuserdata(l, layer_get_map(id));
			return 1;
		}

}



/* set the layer's string list */
int wrap_layer_string_list(lua_State *l)
{
	int id;

	/* setting or reading the string list? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 2 )
		{
			layer_set_string_list(id, lua_touserdata(l, 2));
			return 0;
		}
	else
		{
			lua_pushlightuserdata(l, layer_get_string_list(id));
			return 1;
		}

}



/* set the layer visibility */
int wrap_layer_visible(lua_State *l)
{
	int id, vis;

	/* setting or reading the visibility? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 2 )
		{
			layer_set_visible(id, lua_tonumber(l, 2));
			return 0;
		}
	else
		{
			vis = layer_get_visible(id);
			if( vis == ERR )
				return luaL_error(l, "Invalid layer ");
			else
				{
					lua_pushboolean(l, vis);
					return 1;
				}

		}

}



/* set the layer sorting */
int wrap_layer_sorting(lua_State *l)
{
	int id, sort;

	/* setting or reading the layer sorting? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 2 )
		{
			layer_set_sorting(id, lua_tonumber(l, 2));
			return 0;
		}
	else
		{
			sort = layer_get_sorting(id);
			if( sort == ERR )
				return luaL_error(l, "Invalid layer ");
			else
				{
					lua_pushboolean(l, sort);
					return 1;
				}

		}

}





/* set the camera */
int wrap_layer_camera(lua_State *l)
{
	int id, x, y;

	/* setting or reading the layer camera? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 3 )
		{
			layer_set_camera(id, lua_tonumber(l, 2), lua_tonumber(l, 3));
			return 0;
		}
	else
		{
			if( layer_get_camera(id, &x, &y) == ERR )
				return luaL_error(l, "Invalid layer ");
			else
				{
					lua_pushinteger(l, x);
					lua_pushinteger(l, y);
					return 2;
				}

		}

}



/* set the layer viewport */
int wrap_layer_view(lua_State *l)
{
	int id;
	box view;

	/* setting or reading the layer viewport? */
	id = lua_tonumber(l, 1);
	if( lua_gettop(l) == 5 )
		{
			view.x1 = lua_tonumber(l, 2);
			view.y1 = lua_tonumber(l, 3);
			view.x2 = lua_tonumber(l, 4);
			view.y2 = lua_tonumber(l, 5);
			layer_set_view(id, &view);
			return 0;
		}
	else
		{
			if( layer_get_view(id, &view) == ERR )
				return luaL_error(l, "Invalid layer ");
			else
				{
					lua_pushinteger(l, view.x1);
					lua_pushinteger(l, view.y1);
					lua_pushinteger(l, view.x2);
					lua_pushinteger(l, view.y2);
					return 4;
				}

		}

}
