#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-list.h"






/* creating a new list .. */
int wrap_list_create(lua_State *l)
{
	lua_pushlightuserdata(l, list_create());
	return 1;
}



/* emptying a list. */
int wrap_list_empty(lua_State *l)
{
	list *my_l;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	list_empty(my_l);
	return 0;
}



/* deleting a list. */
int wrap_list_delete(lua_State *l)
{
	list *my_l;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	list_delete(my_l);
	return 0;
}



/* adding to a list. */
int wrap_list_add(lua_State *l)
{
	list *my_l;
	void *item;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	item = lua_touserdata(l, 2);
	list_add(my_l, item);
	return 0;
}



/* prepending to a list. */
int wrap_list_prepend(lua_State *l)
{
	list *my_l;
	void *item;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	item = lua_touserdata(l, 2);
	list_prepend(my_l, item);
	return 0;
}



/* shift from a list. */
int wrap_list_shift(lua_State *l)
{
	list *my_l;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	lua_pushlightuserdata(l, list_shift(my_l));
	return 1;
}




/* pop from a list. */
int wrap_list_pop(lua_State *l)
{
	list *my_l;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	lua_pushlightuserdata(l, list_pop(my_l));
	return 1;
}



/* remove from a list. */
int wrap_list_remove(lua_State *l)
{
	/* the list and item pointer */
	list *my_l;
	void *item;

	/* the direction */
	const char *dir_stg;
	int dir;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	item = lua_touserdata(l, 2);

	/* if the optional direction argument is given, parse it out */
	if( lua_gettop(l) == 3 )
		{
			dir_stg = lua_tostring(l, 3);
			if( !strcmp(dir_stg, "head") )
				dir = LIST_HEAD;
			else if( !strcmp(dir_stg, "tail") )
				dir = LIST_TAIL;
			else if( !strcmp(dir_stg, "all") )
				dir = LIST_ALL;
			else
				return luaL_error(l, "Unknown direction ");
		}
	else
		dir = LIST_ALL;

	/* remove the item and done */
	list_remove(my_l, item, dir);
	return 0;
}



/* get the length of a list. */
int wrap_list_length(lua_State *l)
{
	list *my_l;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	lua_pushinteger(l, list_length(my_l));
	return 1;
}



/* is an item in a list? */
int wrap_list_find(lua_State *l)
{
	list *my_l;
	void *item;

	my_l = lua_touserdata(l, 1);
	if( !my_l )
		return luaL_error(l, "Invalid list ");

	item = lua_touserdata(l, 2);
	lua_pushboolean(l, list_find(my_l, item));
	return 1;
}
