#include <stdio.h>
#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-map.h"






/* creating a new map .. */
int wrap_map_create(lua_State *l)
{
	lua_pushlightuserdata(l, map_create());
	return 1;
}



/* emptying a map .. */
int wrap_map_empty(lua_State *l)
{
	map *map;
	int delete_tiles;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	/* if omitted, it will default to 0 */
	delete_tiles = lua_tonumber(l, 2);

	map_empty(map, delete_tiles);
	return 0;
}



/* deleting a map .. */
int wrap_map_delete(lua_State *l)
{
	map *map;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	map_delete(map);
	return 0;
}



/* set the map dimensions */
int wrap_map_size(lua_State *l)
{
	map *map;
	int w, h;


	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");


	if( lua_gettop(l) == 3 )
		{
			/* set the map dimensions */
			w = lua_tonumber(l, 2);
			h = lua_tonumber(l, 3);
			map_set_size(map, w, h);
			return 0;
		}
	else
		{
			/* read the map size .. */
			if( map_get_size(map, &w, &h) == ERR )
				return luaL_error(l, "Invalid map id ");
			else
				{
					/* and return the result */
					lua_pushinteger(l, w);
					lua_pushinteger(l, h);
					return 2;
				}

		}

}



/* set the map tile size */
int wrap_map_tile_size(lua_State *l)
{
	map *map;
	int tw, th;


	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");


	if( lua_gettop(l) == 3 )
		{
			/* set the map dimensions */
			tw = lua_tonumber(l, 2);
			th = lua_tonumber(l, 3);
			map_set_tile_size(map, tw, th);
			return 0;
		}
	else
		{
			/* read the map size .. */
			if( map_get_tile_size(map, &tw, &th) == ERR )
				return luaL_error(l, "Invalid map id ");
			else
				{
					/* and return the result */
					lua_pushinteger(l, tw);
					lua_pushinteger(l, th);
					return 2;
				}

		}

}




/* set the tile for the given index */
int wrap_map_tile(lua_State *l)
{
	map *map;
	tile *tile;
	int index;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	index = lua_tonumber(l, 2);


	if( lua_gettop(l) == 3 )
		{
			/* set the map dimensions */
			tile = lua_touserdata(l, 3);
			map_set_tile(map, index, tile);
			return 0;
		}
	else
		{
			/* read the tile .. */
			if( map_get_tile(map, index, &tile) == ERR )
				return luaL_error(l, "Invalid map id ");
			else
				{
					lua_pushlightuserdata(l, tile);
					return 1;
				}

		}

}



/* set the map data itself */
int wrap_map_set_data(lua_State *l)
{
	/* map pointer */
	map *map;
	int w, h;

	/* the length of the map-data word-array and the word-array itself */
	size_t len;
	const unsigned char *data;

	/* in the event that the data must be parsed in from a table .. */
	int i;
	short *parsed;


	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");


	/* check the map dimensions */
	if( map_get_size(map, &w, &h) == ERR )
		return luaL_error(l, "Map dimensions not set ");


	/* is it a string of binary data, or is it a table of numbers to be packed? */
	if( lua_isstring(l, 2) )
		{
			/* it is a string of binary data .. */
			data = lua_tolstring(l, 2, &len);

			/* verify the length of the wordarray */
			if( len != w * h * sizeof(short) )
				return luaL_error(l, "Amount of data doesn't match map dimensions ");

			/* load the map */
			map_set_data(map, (short *)data);

		}
	else
		{
			/* verify the length of the table */
			if( lua_objlen(l, 2) != w * h )
				return luaL_error(l, "Amount of data doesn't match map dimensions ");


			/* allocate the tile buffer */
			parsed = ck_malloc(w * h * sizeof(short));

			/* and process the table */
			lua_pushnil(l);
			for( i=0; lua_next(l, 2); i++ )
				{
					*(parsed+i) = (short)lua_tonumber(l, -1);
					lua_pop(l, 1);
				}

			/* load the map */
			map_set_data(map, parsed);

			/* and done! */
			free(parsed);

		}

	return 0;

}





/* set a single element of map data */
int wrap_map_set_single(lua_State *l)
{
	/* map pointer, tile position */
	map *map;
	int x, y;

	/* the data itself, which will be truncated to short */
	int data;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	x = lua_tonumber(l, 2);
	y = lua_tonumber(l, 3);
	data = lua_tonumber(l, 4);

	/* now add the map data */
	map_set_single(map, x, y, (short)data);
	return 0;

}



/* animate all tiles */
int wrap_map_animate_tiles(lua_State *l)
{
	map *map;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	map_animate_tiles(map);
	return 0;
}



/* reset all tile animations */
int wrap_map_reset_tiles(lua_State *l)
{
	map *map;

	map = lua_touserdata(l, 1);
	if( !map )
		return luaL_error(l, "Invalid map ");

	map_reset_tiles(map);
	return 0;
}
