#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-song.h"







/* play a song from a named file */
int wrap_song_play_file(lua_State *l)
{
	const char *file;
	int delay;

	file = lua_tostring(l, 1);

	/* fetch the optional delay argument */
	if( lua_gettop(l) == 2 )
		delay = lua_tonumber(l, 2);
	else
		delay = 0;

	song_play_from_disk(file, delay);
	return 0;
}



/* play a song from a buffer */
int wrap_song_play_buffer(lua_State *l)
{
	/* buffer containing song data */
	const unsigned char *data;
	size_t len;

	int delay;

	data = lua_tolstring(l, 1, &len);

	/* fetch the optional delay argument */
	if( lua_gettop(l) == 2 )
		delay = lua_tonumber(l, 2);
	else
		delay = 0;

	song_play_from_buffer(len, data, delay);
	return 0;

}



/* pause the song */
int wrap_song_pause(lua_State *l)
{
	song_pause();
	return 0;
}




/* resume the song */
int wrap_song_resume(lua_State *l)
{
	song_resume();
	return 0;
}




/* stop the song, with optional fade-out delay */
int wrap_song_halt(lua_State *l)
{
	int delay;

	/* fetch the optional delay argument */
	if( lua_gettop(l) == 1 )
		delay = lua_tonumber(l, 1);
	else
		delay = 0;

	song_stop(delay);
	return 0;
}



/* set the position of the currently-playing song */
int wrap_song_set_position(lua_State *l)
{
	song_set_position(lua_tonumber(l, 1));
	return 0;
}



/* adjust volume of currently-playing song */
int wrap_song_adj_vol(lua_State *l)
{
	song_adjust_vol(lua_tonumber(l, 1));
	return 0;
}
