#include <stdio.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-sound.h"









/* we have been requested to load a sound from a file */
int wrap_sound_load_file(lua_State *l)
{
	const char *file;
	sound *s;

	file = lua_tostring(l, 1);

	/* load the sound */
	s = sound_load_from_disk(file);
	if( !s )
		return luaL_error(l, "Invalid sound ");
	else
		{
			lua_pushlightuserdata(l, s);
			return 1;
		}

}




/* we have been requested to load a sound from a memory buffer */
int wrap_sound_load_buffer(lua_State *l)
{
	const unsigned char *data;
	size_t len;
	sound *s;

	data = lua_tolstring(l, 1, &len);

	/* load the sound */
	s = sound_load_from_buffer(len, data);
	if( !s )
		return luaL_error(l, "Invalid sound ");
	else
		{
			lua_pushlightuserdata(l, s);
			return 1;
		}

}




/*
 * we have been requested to load raw sound data.  note that
 * the sound must consist of unsigned 8-bit samples!
 */
int wrap_sound_load_raw(lua_State *l)
{
	/* buffer data */
	const unsigned char *data;
	size_t len;
	sound *s;

	data = lua_tolstring(l, 1, &len);

	/* load the sound */
	s = sound_load_raw(len, data);
	if( !s )
		return luaL_error(l, "Invalid sound ");
	else
		{
			lua_pushlightuserdata(l, s);
			return 1;
		}

}




/* play the named sound */
int wrap_sound_play(lua_State *l)
{
	sound *sound;   /* ptr to name of sound */
	int vol;       /* volume to use */
	int ch;        /* the channel to return */

	sound = lua_touserdata(l, 1);

	/* was the optional volume level passed in? */
	if( lua_gettop(l) == 2 )
		vol = lua_tonumber(l, 2);
	else
		vol = MIX_MAX_VOLUME;

	/* play the sound */
	ch = sound_play(sound, vol);

	/* and return the channel id */
	lua_pushinteger(l, ch);
	return 1;

}




/* halt the requested audio channel */
int wrap_sound_halt(lua_State *l)
{
	sound_halt(lua_tonumber(l, 1));
	return 0;
}



/* adjust the volume on the requested audio channel */
int wrap_sound_adj_vol(lua_State *l)
{
	sound_adjust_vol(lua_tonumber(l, 1), lua_tonumber(l, 2));
	return 0;
}



/* adjust the panning on the requested audio channel */
int wrap_sound_adj_pan(lua_State *l)
{
	sound_adjust_pan(lua_tonumber(l, 1), lua_tonumber(l, 2));
	return 0;
}
