#include <stdio.h>
#include <stdlib.h>
#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>
#include <brick.h>
#include "lua-mod.h"
#include "lua-mod-sprite.h"





/* create a new sprite */
int wrap_sprite_create(lua_State *l)
{
	lua_pushlightuserdata(l, sprite_create());
	return 1;
}




/* clone the sprite */
int wrap_sprite_copy(lua_State *l)
{
	sprite *sprite;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	lua_pushlightuserdata(l, sprite_copy(sprite));
	return 1;
}




/* delete the given sprite */
int wrap_sprite_delete(lua_State *l)
{
	sprite *sprite;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	sprite_delete(sprite);
	return 0;
}




/* set the visible sprite frame */
int wrap_sprite_frame(lua_State *l)
{
	sprite *sprite;
	int index;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	if( lua_gettop(l) == 2 )
		{
			index = lua_tonumber(l, 2);
			sprite_set_frame(sprite, index);
			return 0;
		}
	else
		{
			if( sprite_get_frame(sprite, &index) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					lua_pushinteger(l, index);
					return 1;
				}
		}

}




/* set the sprite scale */
int wrap_sprite_scale(lua_State *l)
{
	sprite *sprite;
	int sx, sy;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	if( lua_gettop(l) == 3 )
		{
			sx = lua_tonumber(l, 2) * 65536;
			sy = lua_tonumber(l, 3) * 65536;
			sprite_set_scale(sprite, sx, sy);
			return 0;
		}
	else
		{
			if( sprite_get_scale(sprite, &sx, &sy) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					lua_pushnumber(l, (float)sx/65536);
					lua_pushnumber(l, (float)sy/65536);
					return 2;
				}
		}

}




/* the sprite's z hint */
int wrap_sprite_z_hint(lua_State *l)
{
	sprite *sprite;
	int z_hint;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	if( lua_gettop(l) == 2 )
		{
			z_hint = lua_tonumber(l, 2);
			sprite_set_z_hint(sprite, z_hint);
			return 0;
		}
	else
		{
			if( sprite_get_z_hint(sprite, &z_hint) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					lua_pushinteger(l, z_hint);
					return 1;
				}
		}

}




/* the sprite's collision mode */
int wrap_sprite_collides(lua_State *l)
{
	/* collision modes */
	const char *mode, *modes[] = { "off", "box", "pixel" };
	int idx;
	sprite *sprite;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	if( lua_gettop(l) == 2 )
		{
			/* set the collision type */
			mode = lua_tostring(l, 2);

			if( !strcmp(mode, "off") )
				sprite_set_collides(sprite, COLLISION_OFF);
			else if( !strcmp(mode, "box") )
				sprite_set_collides(sprite, COLLISION_BOX);
			else if( !strcmp(mode, "pixel") )
				sprite_set_collides(sprite, COLLISION_PIXEL);
			else
				return luaL_error(l, "Unknown collision mode ");

			return 0;
		}
	else
		{
			/* read and return the name of the animation type */
			if( sprite_get_collides(sprite, &idx) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					lua_pushstring(l, modes[idx]);
					return 1;
				}

		}

}





/* set the sprite frame bounding box */
int wrap_sprite_bounding_box(lua_State *l)
{
	sprite *sprite;
	int index;
	box b;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	index = lua_tonumber(l, 2);
	b.x1 = lua_tonumber(l, 3);
	b.y1 = lua_tonumber(l, 4);
	b.x2 = lua_tonumber(l, 5);
	b.y2 = lua_tonumber(l, 6);

	sprite_set_bounding_box(sprite, index, &b);
	return 0;
}





/* set the pixel mask */
int wrap_sprite_set_pixel_mask(lua_State *l)
{
	/* sprite pointer and frame index */
	sprite *sprite;
	int index;

	/* pixel data buffer */
	const unsigned char *data;
	size_t len;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	index = lua_tonumber(l, 2);
	data = lua_tolstring(l, 3, &len);

	sprite_set_pixel_mask(sprite, index, data);
	return 0;
}





/* set the sprite pixel mask from a source frame */
int wrap_sprite_set_pixel_mask_from(lua_State *l)
{
	/* sprite pointer and frame index */
	sprite *sprite;
	int index;
	frame *frame;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	index = lua_tonumber(l, 2);

	frame = lua_touserdata(l, 3);
	if( !frame )
		return luaL_error(l, "Invalid frame ");

	sprite_set_pixel_mask_from(sprite, index, frame);
	return 0;
}




/* set the sprite position */
int wrap_sprite_position(lua_State *l)
{
	sprite *sprite;
	int x, y;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");


	if( lua_gettop(l) == 3 )
		{
			/* set the position */
			x = lua_tonumber(l, 2);
			y = lua_tonumber(l, 3);

			sprite_set_position(sprite, x, y);
			return 0;
		}
	else
		{
			/* get the position? */
			if( sprite_get_position(sprite, &x, &y) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					/* and return the x and y values */
					lua_pushinteger(l, x);
					lua_pushinteger(l, y);
					return 2;
				}

		}

}





/* set the sprite velocity */
int wrap_sprite_velocity(lua_State *l)
{
	sprite *sprite;
	int x, y;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");


	if( lua_gettop(l) == 3 )
		{
			/* set the velocity */
			x = lua_tonumber(l, 2);
			y = lua_tonumber(l, 3);

			sprite_set_velocity(sprite, x, y);
			return 0;
		}
	else
		{
			/* get the velocity? */
			if( sprite_get_velocity(sprite, &x, &y) == ERR )
				return luaL_error(l, "Invalid sprite id ");
			else
				{
					/* and return the x and y values */
					lua_pushinteger(l, x);
					lua_pushinteger(l, y);
					return 2;
				}

		}

}




/* set a sprite frame .. */
int wrap_sprite_add_frame(lua_State *l)
{
	sprite *sprite;
	frame *frame;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	frame = lua_touserdata(l, 2);
	if( !frame )
		return luaL_error(l, "Invalid frame ");

	lua_pushinteger(l, sprite_add_frame(sprite, frame));
	return 1;
}





/* add frame data to a sprite */
int wrap_sprite_add_frame_data(lua_State *l)
{
	/* sprite pointer */
	sprite *sprite;

	/* data to retrieve from the lua arguments */
	const char *mode;
	int w,h;
	const unsigned char *data; /* stored in a lua string */
	size_t len;                /* length of data */

	/* also cumbersome - variables to read and parse convolution kernel or lookup table */
	convolution ck;
	lut ldata;

	int i;


	/* the result */
	int res;

	/* check for some args */
	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	mode = lua_tostring(l, 2);
	w = lua_tonumber(l, 3);
	h = lua_tonumber(l, 4);

	/* and parse out based on mode */
	if( !strcmp(mode, "none") )
		res = sprite_add_frame_data(sprite, FRAME_NONE, w, h, NULL, NULL);
	else
		{
			/* one of the non-null sprite types */
			data = lua_tolstring(l, 5, &len);

			if( !strcmp(mode, "rgba") )
				res = sprite_add_frame_data(sprite, FRAME_RGBA, w, h, data, NULL);
			else if( !strcmp(mode, "rgb") )
				res = sprite_add_frame_data(sprite, FRAME_RGB, w, h, data, NULL);
			else if( !strcmp(mode, "br") )
				res = sprite_add_frame_data(sprite, FRAME_BR, w, h, data, NULL);
			else if( !strcmp(mode, "ct") )
				res = sprite_add_frame_data(sprite, FRAME_CT, w, h, data, NULL);
			else if( !strcmp(mode, "hl") )
				res = sprite_add_frame_data(sprite, FRAME_HL, w, h, data, NULL);
			else if( !strcmp(mode, "sl") )
				res = sprite_add_frame_data(sprite, FRAME_SL, w, h, data, NULL);
			else if( !strcmp(mode, "sat") )
				res = sprite_add_frame_data(sprite, FRAME_SAT, w, h, data, NULL);
			else if( !strcmp(mode, "displ") )
				res = sprite_add_frame_data(sprite, FRAME_DISPL, w, h, data, NULL);
			else if( !strcmp(mode, "convo") )
			  {
					/*
					 * retrieve the convolution-kernel data from the arguments thus:
					 * w h [ k1 k2 k3 .. kn ] divisor offset
					 */

					/* first fetch the kernel width and height */
					ck.kw = lua_tonumber(l, 6);
					ck.kh = lua_tonumber(l, 7);

					if( ck.kw > MAX_CK_SIZE || ck.kh > MAX_CK_SIZE )
						return luaL_error(l,  "Convolution kernel too large ");

					/* convolution kernel table is in the stack at index 7 */
					lua_pushnil(l);
					for( i=0; lua_next(l, 8); i++ )
						{
						  ck.kernel[i] = (char)lua_tonumber(l, -1);
							lua_pop(l, 1);
						}

					ck.divisor = lua_tonumber(l, 9);
					ck.offset = lua_tonumber(l, 10);

					if( i != ck.kw * ck.kh )
					  return luaL_error(l, "Incorrect amount of kernel data provided ");

					/* at last, add the convolution kernel */
					res = sprite_add_frame_data(sprite, FRAME_CONVO, w, h, data, &ck);

			  }
			else if( !strcmp(mode, "lut") )
			  {
					/* retrieve the lookup table data from three provided tables */

					/* red table is in the stack at index 5 */
					lua_pushnil(l);
					for( i=0; lua_next(l, 6) && i < RGB_RANGE; i++ )
						{
							ldata.r[i] = (unsigned char)lua_tonumber(l, -1);
							lua_pop(l, 1);
					  }

					/* green table is in the stack at index 6 */
					lua_pushnil(l);
					for( i=0; lua_next(l, 7) && i < RGB_RANGE; i++ )
						{
							ldata.g[i] = (unsigned char)lua_tonumber(l, -1);
							lua_pop(l, 1);
					  }

					/* blue table is in the stack at index 7 */
					lua_pushnil(l);
					for( i=0; lua_next(l, 8) && i < RGB_RANGE; i++ )
						{
							ldata.b[i] = (unsigned char)lua_tonumber(l, -1);
							lua_pop(l, 1);
					  }

					/* at last, add the convolution kernel */
					res = sprite_add_frame_data(sprite, FRAME_LUT, w, h, data, &l);
			  }
			else if( !strcmp(mode, "xor") )
				res = sprite_add_frame_data(sprite, FRAME_XOR, w, h, data, NULL);
			else
			  return luaL_error(l, "Unknown frame type ");
		}


	/* and finished */
	if( res < 0 )
		{
			switch(res)
				{
					case ERR: return luaL_error(l, "Invalid sprite id ");
					case ERR_BAD_FRAME_TYPE: return luaL_error(l, "Unknown frame type ");
					default: return luaL_error(l, "Unknown error ");
				}
		}
	else
		{
			lua_pushinteger(l, res);
			return 1;
		}

}





/* add a sprite subframe .. */
int wrap_sprite_add_subframe(lua_State *l)
{
	sprite *sprite;
	int index;
	frame *frame;

	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	index = lua_tonumber(l, 2);

	frame = lua_touserdata(l, 3);
	if( !frame )
		return luaL_error(l, "Invalid frame ");

	lua_pushinteger(l, sprite_add_subframe(sprite, index, frame));
	return 1;
}





/* set the sprite's motion control program */
int wrap_sprite_load_program(lua_State *l)
{
	/* sprite pointer */
	sprite *sprite;

	/* motion control program */
	const char *mcp;

	/* result */
	int res;


	/* retrieve the sprite and the motion control program */
	sprite = lua_touserdata(l, 1);
	if( !sprite )
		return luaL_error(l, "Invalid sprite ");

	mcp = lua_tostring(l, 2);

	/* load the program */
	res = sprite_load_program(sprite, mcp);

	/* was there an error? */
	if( res < 0 )
		{
			/* oh no!  an error! */
			switch(res)
				{
					case ERR: return luaL_error(l, "Invalid sprite id ");
					case ERR_BAD_INST: return luaL_error(l, "Unknown instruction ");
					case ERR_BAD_VAR: return luaL_error(l, "Unknown variable ");
					case ERR_BAD_ARG: return luaL_error(l, "Unknown argument ");
					case ERR_TOO_LONG: return luaL_error(l, "Motion-control program is too long ");
					default: return luaL_error(l, "Unknown error ");
				}
		}

	/* success! */
	return 0;

}
