/*
 * some function prototypes
 */

/* how many sprite collisions will the module accept? */
#define MAX_SPRITE_COLLISIONS 40

/* how long is pointer, when converted to text? */
#define PTR_LEN 20

/* pulled from SDL_mixer.h */
#define MIX_MAX_VOLUME 128


/* debugging-output macros */
#define DEBUG(a)      do { fprintf(stderr,a); } while(0)
#define DEBUG_N(a, b) do { fprintf(stderr,a,b); } while(0)
#define DEBUGF        do { fprintf(stderr,"success!\n"); } while(0)
#define DEBUGNF       do { fprintf(stderr,"failure.\n"); } while(0)



/* all our stuff for making the C and lua interact... */
int luaopen_brick (lua_State *);

extern int wrap_graphics_open(lua_State *);
extern int wrap_graphics_close(lua_State *);
extern int wrap_graphics_window_title(lua_State *);

extern int wrap_audio_modes(lua_State *);
extern int wrap_audio_open(lua_State *);
extern int wrap_audio_close(lua_State *);

extern int wrap_io_fetch(lua_State *);
extern int wrap_io_mouse(lua_State *);
extern int wrap_io_has_quit(lua_State *);
extern int wrap_io_grab(lua_State *);
extern int wrap_io_wait(lua_State *);
extern int wrap_io_assign(lua_State *);
extern int wrap_io_read_key(lua_State *);

extern int wrap_render_bg_fill(lua_State *);
extern int wrap_render_bg_color(lua_State *);
extern int wrap_render_set_overdraw(lua_State *);
extern int wrap_render_display(lua_State *);
extern int wrap_render_to_disk(lua_State *);

extern int wrap_font_add(lua_State *);
extern int wrap_font_from_disk(lua_State *);
extern int wrap_font_from_buffer(lua_State *);

extern int wrap_sound_load_file(lua_State *);
extern int wrap_sound_load_buffer(lua_State *);
extern int wrap_sound_load_raw(lua_State *);
extern int wrap_sound_play(lua_State *);
extern int wrap_sound_halt(lua_State *);
extern int wrap_sound_adj_vol(lua_State *);
extern int wrap_sound_adj_pan(lua_State *);

extern int wrap_song_play_file(lua_State *);
extern int wrap_song_play_buffer(lua_State *);
extern int wrap_song_pause(lua_State *);
extern int wrap_song_resume(lua_State *);
extern int wrap_song_halt(lua_State *);
extern int wrap_song_set_position(lua_State *);
extern int wrap_song_adj_vol(lua_State *);

extern int wrap_list_create(lua_State *);
extern int wrap_list_empty(lua_State *);
extern int wrap_list_delete(lua_State *);
extern int wrap_list_add(lua_State *);
extern int wrap_list_prepend(lua_State *);
extern int wrap_list_shift(lua_State *);
extern int wrap_list_pop(lua_State *);
extern int wrap_list_remove(lua_State *);
extern int wrap_list_length(lua_State *);
extern int wrap_list_find(lua_State *);

extern int wrap_layer_add(lua_State *);
extern int wrap_layer_swap(lua_State *);
extern int wrap_layer_remove(lua_State *);
extern int wrap_layer_copy(lua_State *);
extern int wrap_layer_sprite_list(lua_State *);
extern int wrap_layer_map(lua_State *);
extern int wrap_layer_string_list(lua_State *);
extern int wrap_layer_visible(lua_State *);
extern int wrap_layer_sorting(lua_State *);
extern int wrap_layer_camera(lua_State *);
extern int wrap_layer_view(lua_State *);

extern int wrap_frame_info(lua_State *);
extern int wrap_frame_create(lua_State *);
extern int wrap_frame_delete(lua_State *);
extern int wrap_frame_copy(lua_State *);
extern int wrap_frame_set_offset(lua_State *);
extern int wrap_frame_set_mask(lua_State *);
extern int wrap_frame_set_mask_from(lua_State *);
extern int wrap_frame_slice(lua_State *);
extern int wrap_frame_convert(lua_State *);
extern int wrap_frame_effect(lua_State *);
extern int wrap_frame_from_disk(lua_State *);
extern int wrap_frame_from_buffer(lua_State *);

extern int wrap_map_create(lua_State *);
extern int wrap_map_empty(lua_State *);
extern int wrap_map_delete(lua_State *);
extern int wrap_map_size(lua_State *);
extern int wrap_map_tile_size(lua_State *);
extern int wrap_map_tile(lua_State *);
extern int wrap_map_set_data(lua_State *);
extern int wrap_map_set_single(lua_State *);
extern int wrap_map_animate_tiles(lua_State *);
extern int wrap_map_reset_tiles(lua_State *);

extern int wrap_tile_create(lua_State *);
extern int wrap_tile_delete(lua_State *);
extern int wrap_tile_anim_type(lua_State *);
extern int wrap_tile_collides(lua_State *);
extern int wrap_tile_add_frame(lua_State *);
extern int wrap_tile_add_frame_data(lua_State *);
extern int wrap_tile_set_pixel_mask(lua_State *);
extern int wrap_tile_set_pixel_mask_from(lua_State *);
extern int wrap_tile_animate(lua_State *);
extern int wrap_tile_reset(lua_State *);

extern int wrap_sprite_create(lua_State *);
extern int wrap_sprite_copy(lua_State *);
extern int wrap_sprite_delete(lua_State *);
extern int wrap_sprite_frame(lua_State *);
extern int wrap_sprite_scale(lua_State *);
extern int wrap_sprite_z_hint(lua_State *);
extern int wrap_sprite_collides(lua_State *);
extern int wrap_sprite_bounding_box(lua_State *);
extern int wrap_sprite_set_pixel_mask(lua_State *);
extern int wrap_sprite_set_pixel_mask_from(lua_State *);
extern int wrap_sprite_position(lua_State *);
extern int wrap_sprite_velocity(lua_State *);
extern int wrap_sprite_add_frame(lua_State *);
extern int wrap_sprite_add_frame_data(lua_State *);
extern int wrap_sprite_add_subframe(lua_State *);
extern int wrap_sprite_load_program(lua_State *);

extern int wrap_string_create(lua_State *);
extern int wrap_string_delete(lua_State *);
extern int wrap_string_get_box(lua_State *);
extern int wrap_string_set_font(lua_State *);
extern int wrap_string_set_position(lua_State *);
extern int wrap_string_set_text(lua_State *);

extern int wrap_inspect_adjacent_tiles(lua_State *);
extern int wrap_inspect_obscured_tiles(lua_State *);
extern int wrap_inspect_line_of_sight(lua_State *);
extern int wrap_inspect_in_frame(lua_State *);
extern int wrap_inspect_near_point(lua_State *);

extern int wrap_collision_map(lua_State *);
extern int wrap_collision_sprites(lua_State *);

extern int wrap_motion_list(lua_State *);
extern int wrap_motion_single(lua_State *);

extern int wrap_clock_ms(lua_State *);
extern int wrap_clock_wait(lua_State *);
