/*
 * brick.vapi
 *
 * Copyright (c) 2006, 2007, 2008, 2009, 2010, 2011 Stephen M. Havelka
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * For more information, please visit the project web site:
 * http://rs.tc/br/
 *
 */

[CCode (lower_case_cprefix = "", cheader_filename = "brick.h")]
namespace Brick {

	/*
	 * Enumerations used throughout the brick engine
	 */
	[CCode (cname = "int", cprefix = "LIST_")]
	public enum List_mode { HEAD, TAIL, ALL }

	[CCode (cname = "int", cprefix = "GRAPHICS_")]
	public enum Graphics_mode { SDL, ACCEL, WINDOWED, FS, 0, 90, 180, 270 }

	[CCode (cname = "int", cprefix = "AUDIO_")]
	public enum Audio_mode { OFF, SPEAKER }

	[CCode (cname = "int", cprefix = "FRAME_")]
	public enum Frame_type { NONE, RGBA, RGB, HL, SL, BR, CT, SAT, DISPL, CONVO, LUT, XOR }

	[CCode (cname = "int", cprefix = "FRAME_EFFECT_")]
	public enum Frame_effect_type { DROP_SHADOW }

	[CCode (cname = "int", cprefix = "ANIMATE_")]
	public enum Animate { STILL, FWD, REV, PP, PP_REV }

	[CCode (cname = "int", cprefix = "COLLISION_")]
	public enum Collision_mode { OFF, BOX, PIXEL }
	public enum Collision_result { ATSTART, NEVER, INMOTION }

	[CCode (cname = "int", cprefix = "INSPECT_")]
	public enum Inspect_dir { NW, N, NE, E, SE, S, SW, W }

	[CCode (cname = "int", cprefix = "IO_")]
	public enum Input_dir { UP, RIGHT, DOWN, LEFT }

	[CCode (cname = "int", cprefix = "IO_")]
	public enum Input_type { AXIS, HAT, BUTTON }

	[CCode (cname = "int", cprefix = "EVENT_")]
	public enum Event_msg { GO, STOP, PAUSE, SKIP1 }

	[CCode (cname = "int", cprefix = "ERR_")]
	public enum Error {
		[CCode (cname = "ERR")]
		FAIL,
		CANT_REOPEN, BAD_MODE, BAD_FILE,
		SDL_FAILED, SDL_VIDEO_FAILED, SDL_MIXER_FAILED,
		BAD_FRAME_TYPE,
		INVALID_FILE,
		BAD_INST, BAD_VAR, BAD_ARG, TOO_LONG, BAD_LIST, BAD_INST_BC, BAD_ARG_BC
	}


	[CCode (cprefix = "", lower_case_cprefix = "")]
	namespace Config {
		[CCode (cname = "BRICK_VERSION")]
		public const string VERSION;

		public const int WITH_GL;
		public const int WITH_SIMD;
		public const int WITH_IMAGE;
		public const int MAX_WIDTH;
		public const int MAX_HEIGHT;

		public const int MAX_MCP_LENGTH;

		public const int MAX_STRING_LENGTH;

		public const int DISPL_SPAN;

		public const int FONT_CT;

		public const int MAX_TILES;

		public const int MAX_CK_SIZE;

		public const int MAX_JOY;
		public const int MAX_AXES;
		public const int MAX_HATS;
		public const int MAX_BUTTONS;
		public const int MAX_MOUSE;
		public const int MAX_MOUSE_BUTTONS;

		public const int AUDIO_RATE;
		public const int AUDIO_FORMAT;
		public const int AUDIO_CHANNELS;
		public const int AUDIO_BUFFERS;

		public const int RGB_BITS;
		public const int RGB_BYTES;

		public const int RGBA_BITS;
		public const int RGBA_BYTES;

		public const int RGB_RANGE;
		public const int RGB_MAX;

	}



	/*
	 * Data structures used throughout the engine
	 */
	[CCode (cname = "point", destroy_function = "")]
	public struct Point {
		public int x;
		public int y;
	}

	[CCode (cname = "vector", destroy_function = "")]
	public struct Vector {
		public int x;
		public int y;
	}

	[CCode (cname = "dimensions", destroy_function = "")]
	public struct Dimensions {
		public int w;
		public int h;
	}

	[CCode (cname = "box", destroy_function = "")]
	public struct Box {
		public int x1;
		public int x2;
		public int y1;
		public int y2;
	}

	[CCode (cname = "color", destroy_function = "")]
	public struct Color {
		public uint8 r;
		public uint8 g;
		public uint8 b;
		public uint8 a;
	}

	[CCode (cname = "convolution", destroy_function = "")]
	public struct Convolution {
		public int kw;
		public int kh;
		public int8 kernel[49];  // Set in Brick.Config.MAX_CK_SIZE
		public int divisor;
		public int offset;
	}

	[CCode (cname = "lut", destroy_function = "")]
	public struct Lut {
		public uint8 r[256];  // Set in Brick.Config.RGB_RANGE
		public uint8 g[256];
		public uint8 b[256];
	}

	[CCode (cname = "map_fragment", destroy_function = "")]
	public struct Map_fragment {
		public int w;
		public int h;
		public uint16[] tiles;
	}

	[CCode (cname = "map_collision", destroy_function = "")]
	public struct Map_collision {
		public Collision_mode mode;
		public Vector stop;
		public Vector go;
	}

	[CCode (cname = "sprite_collision", destroy_function = "")]
	public struct Sprite_collision {
		public Collision_mode mode;
		public Vector dir;
		public Vector stop;
		public Sprite target;
	}

	[SimpleType]
	[CCode (cname = "input", destroy_function = "")]
	public struct Input_res {
		public int axis[8];      // Set in Brick.Config.MAX_AXES
		public Vector hat[4];    // Set in Brick.Config.MAX_HATS
		public int button[20];   // Set in Brick.Config.MAX_BUTTONS
		public int space;
		public int tab;
		public int sel;
		public int pause;
		public int esc;
	}

	[CCode (cname = "mouse", destroy_function = "")]
	public struct Mouse_res {
		public int x;
		public int y;
		public int button[7];
	}



	/*
	 * The brick engine routines, bundled into namespace and class as appropriate
	 */
	[CCode (cname = "init_brick")]
	public static void init();

	[CCode (cname = "quit_brick")]
	public static void quit();


	[CCode (cprefix = "graphics_")]
	namespace Graphics {
		public static Error open(int w, int h, int zf, int rot, int flags);
		public static void close();
	}


	[CCode (cprefix = "audio_")]
	namespace Audio {
		public static Error open(Audio_mode mode);
		public static void close();
	}


	[CCode (cprefix = "io_")]
	namespace Io {

		public static Error fetch(int num, out Input_res res);
		public static Error mouse(int num, out Mouse_res res);

		public static Error wait(int fps_delay);
		public static void grab(int mode);
		public static int has_quit();

		public static void assign(int idx, Input_type t, ...);
		public static int read_key();

	}



	[CCode (cprefix = "render_")]
	namespace Render {

		public static void set_bg_fill(int active);
		public static void set_bg_color(uint8 r, uint8 g, uint8 b);
		public static void set_overdraw(int w, int h);

		public static void display();
		public static Error to_disk(string filename);

	}


	[CCode (cprefix = "font_")]
	namespace Font {
		public static void add(string name, int charw, int charh, uint8[] data, Color? key);

		// hmm
		public static void from_disk(string name, string filename, Color? key);
		public static void from_buffer(string name, int len, uint8[] buf, Color? key);
	}



	[Compact]
	[CCode (cname = "sound", cprefix = "sound_")]
	public class Sound {

		[CCode (cname = "sound_load_from_disk")]
		public Sound.load_from_disk(string filename);
		[CCode (cname = "sound_load_from_buffer")]
		public Sound.load_from_buffer(int len, uint8[] buf);
		[CCode (cname = "sound_load_raw")]
		public Sound.load_raw(int len, uint8[] buf);

		public int play(int vol);
		public static void halt(int chan);

		public static void adjust_vol(int chan, int vol);
		public static void adjust_pan(int chan, int pan);

	}


	[CCode (cprefix = "song_")]
	namespace Song {

		public static void play_from_disk(string filename, int fade_in_delay);
		public static void play_from_buffer(int len, uint8[] buf, int fade_in_delay);

		public static void stop(int fade_out_delay);
		public static void pause();
		public static void resume();

		public static void set_position(int pos);
		public static void adjust_vol(int vol);

	}



	[Compact]
	[CCode (cname = "list", cprefix = "list_", free_function = "list_delete")]
	public class List<T> {

		[CCode (cname = "create")]
		public List();

		public void empty();

		public void add(T item);
 		public void prepend(T item);

		public T shift();
		public T pop();
		public void remove(T item, List_mode mode);

		public int length();
		public int find(T item);

		public void sort(comparison_cb c);
		public delegate int comparison_cb(void* a, void* b);

	}


	[CCode (cname = "int", cprefix = "layer_", free_function = "layer_remove")]
	[SimpleType]
	public struct Layer : int {

		[CCode (cname = "layer_add")]
		public Layer ();

		public unowned Layer copy();

		public static int count();

		public void reorder (int arg);

		public unowned List<Sprite> get_sprite_list();
		public unowned Map get_map();
		public unowned List<String> get_string_list();

		public int get_visible();
		public int get_sorting();
		public void get_view(out Box view);

		public void set_sprite_list(List<Sprite> l);
		public void set_map(Map m);
		public void set_string_list(List<String> l);

		public void set_visible(int mode);
		public void set_sorting(int mode);
		public void set_view(Box view);

		public void get_camera(out int x, out int y);
		public void set_camera(int x, int y);
		public void adjust_camera(int x, int y);

	}


	[Compact]
	[CCode (cname = "frame", cprefix = "frame_", free_function = "frame_delete")]
	public class Frame {

		[CCode (cname = "frame_create")]
		public Frame(Frame_type type, int w, int h, void* data, void* aux);

		public unowned Frame copy();

		// hmm
		[CCode (cname = "frame_from_disk")]
		public Frame.from_disk(string filename, Color? key);
		[CCode (cname = "frame_from_buffer")]
		public Frame.from_buffer(int len, uint8[] buf, Color? key);

		public void info(out int w, out int h, out Frame_type type);
		public Frame slice(int x, int y, int w, int h);

		public void set_offset(int x, int y);
		public void set_mask(uint8[] data);
		public void set_mask_from(Frame src);
		public void convert(Frame_type type, void* aux);

		public unowned Frame effect(int type, ...);

	}


	[Compact]
	[CCode (cname = "map", cprefix = "map_", free_function = "map_delete")]
	public class Map {

		[CCode (cname = "map_create")]
		public Map();

		public void empty(int delete_tiles);

		public void get_tile_size(out int w, out int h);
		public void get_size(out int w, out int h);
		public void get_tile(int idx, out Tile t);

		public void set_tile_size(int w, int h);
		public void set_size(int w, int h);
		public void set_tile(int idx, Tile? t);

		public void set_data(uint16[] data);
		public void set_single(int x, int y, uint16 single);

		public void animate_tiles();
		public void reset_tiles();

	}


	[Compact]
	[CCode (cname = "tile", cprefix = "tile_", free_function = "tile_delete")]
	public class Tile {

		[CCode (cname = "tile_create")]
		public Tile();

		public int add_frame(Frame f);
		public int add_frame_data(Frame_type type, int w, int h, void* data, void* aux);

		public void get_collides(out Collision_mode mode);
		public void get_anim_type(out Animate type);

		public void set_collides(Collision_mode mode);
		public void set_anim_type(Animate type);

		public void set_pixel_mask(int idx, uint8[] data);
		public void set_pixel_mask_from(int idx, Frame f);

		public void animate();
		public void reset();

	}


	[Compact]
	[CCode (cname = "sprite", cprefix = "sprite_", free_function = "sprite_delete")]
	public class Sprite {

		[CCode (cname = "sprite_create")]
		public Sprite();

		public unowned Sprite copy();

		public void get_frame(out int frame);
		public void get_z_hint(out int z_hint);
		public void get_collides(out Collision_mode mode);
		public void get_position(out int x, out int y);
		public void get_velocity(out int x, out int y);
		public void get_scale(out int x, out int y);

		public void set_frame(int frame);
		public void set_z_hint(int z_hint);
		public void set_collides(Collision_mode mode);
		public void set_position(int x, int y);
		public void set_velocity(int x, int y);
		public void set_scale(int x, int y);

		public void set_bounding_box(int idx, Box b);
		public void set_pixel_mask(int idx, uint8[] data);
		public void set_pixel_mask_from(int idx, Frame f);
		public int add_frame(Frame f);
		public int add_frame_data(Frame_type type, int w, int h, void* data, void* aux);
		public int add_subframe(int idx, Frame f);
		public Error load_program(string program);

	}


	[Compact]
	[CCode (cname = "string", cprefix = "string_", free_function = "string_delete")]
	public class String {

		[CCode (cname = "string_create")]
		public String();

		public int get_box(string name, out int w, out int h);
		public void set_font(string name);
		public void set_position(int x, int y);
		public void set_text(string text);

	}



	[CCode (cprefix = "motion_")]
	namespace Motion {
		public static Error exec_single(Sprite s);
		public static Error exec_list(List l);
	}



	[CCode (cprefix = "inspect_")]
	namespace Inspect {
		public static void adjacent_tiles(Map m, Sprite s, Inspect_dir d, Map_fragment res);
		public static void obscured_tiles(Map m, Sprite s, Map_fragment res);
		public static int line_of_sight(Map m, Sprite s, int xofs, int yofs, int dist, Sprite tgt);
		public static List in_frame(List l, Box b);
		public static List near_point(List l, int x, int y, int dist);
	}


	[CCode (cprefix = "collision_")]
	namespace Collision {
		public static void with_map(Sprite s, Map m, int slip, out Map_collision res);
		public static int with_sprites(Sprite s, List l, int limit, out Sprite_collision[] res);
	}


	[CCode (cprefix = "clock_")]
	namespace Clock {
		public static int ms();
		public static int wait(int fps);
	}


	[CCode (cprefix = "event_")]
	namespace Event {
		public static int add(int delay, int count, event_cb ev, void* data);
		public static void message(int id, Event_msg msg);

		public delegate void event_cb(void* arg);
	}



}
